<?php

/*

 $Id: range.php,v 1.59 2004/06/29 19:32:54 philips Exp $
 $Source: /data/cvsroot/maintain/modules/class/range.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*!
	@header Range Class
	The range class definition that describes the IP ranges that
	can be used in a zone.

*/

class Range {
	var $id;
	// IP objects
	var $start;
	var $end;

	var $default_workgroup;
	var $default_domain;
	var $type;
	var $subnet;
	var $enabled;
	var $allow_all_hosts;
	
	//array of zones that this range is in (zone ids)
	var $zones;

	var $comment;
	var $total_ips; //total number of ips in this range

	// we can populate this stuff from the database -- no sense in
	//   passing it in.
	function Range ( $range_id = 0 ) {
		global $conf;

		$this->id = $range_id;

		$info = $this->get_info();
		if ($info->id) {
			$ip = new IP();

			$this->start		= new IP($info->start);
			$this->end			= new IP($info->end);
			$this->type			= $info->type;
			$this->zones		= $info->zones;
			$this->subnet		= $info->subnet;
			$this->enabled		= $info->enabled;
			$this->comment 		= $info->comment;
			$this->allow_all_hosts 		= $info->allow_all_hosts;

			//Objects that have to be generated
			$host = new Host(0);
			$results = $host->get_workgroups($info->start);
			$workgroup = @array_shift($results);
			$this->default_workgroup = $workgroup->id;

			$this->default_domain    = new Domain($info->default_domain);

			if ($this->type == "static") {
			$this->total_ips  	 = $this->end->ip_int - $this->start->ip_int + 1 ;
			}
			elseif ($this->type == "dynamic" or $this->type == "ddns") {
			$this->total_ips 	 = intval(($this->end->ip_int - $this->start->ip_int) * $conf['dynamic_overflow_multiple']);
			}

		} //end if info

	} // Range


	function get_info () {
		global $conf;

		$sql = "SELECT * FROM ranges WHERE id='$this->id'";
		$db_result = mysql_query($sql, $conf['dbh']);
		
		$info = mysql_fetch_object($db_result);

		$sql = "SELECT zone,default_domain FROM zone_range WHERE range='$this->id'";
		$db_results = mysql_query($sql, $conf['dbh']);
		
		while ($r = mysql_fetch_object($db_results)) {
			$zones[] = $r->zone;
			if (!$info->default_domain) { $info->default_domain = $r->default_domain; } 
			if ($r->zone == $conf['zone']->id) {
				$info->default_domain = $r->default_domain;
			}

		}


		$info->zones = $zones;

		return $info;
	} // get_info

	//get the comment for a specified zone
	function get_comment($zone_id = 0 ) {
		global $conf;

		if (!$zone_id) { $zone_id = $conf['zone']->id; }

		$sql = "SELECT comment FROM zone_range WHERE range='$this->id' AND zone='$zone_id'";
		$db_results = mysql_query($sql, $conf['dbh']);

		$results = mysql_fetch_row($db_results);

		return $results[0];

	} //get_comment

	//get's the subnet that this range should be in
	function get_subnet($ip_start=0) {
		global $conf;
		
		//if we don't set use the objects
		if (!$ip_start) { $ip_start = $this->start->ip_int; }

                //Get the correct subnet for this
                $sql = "SELECT id FROM subnet WHERE subnet<='$ip_start' ORDER BY subnet DESC LIMIT 1";
                $db_results = mysql_query($sql, $conf['dbh']);
                $results = mysql_fetch_row($db_results);

		return $results[0];


	} //get_subnet

	/*!
		@function get_represented_domains
		@discussion Gets all domains that are inside $this range.
	*/
	function get_represented_domains() {
		global $conf;

		if ($this->type == 'static') {
			$sql = "SELECT DISTINCT(host.domain) FROM host WHERE host.ip>='" . $this->start->ip_int . "' AND host.ip<='" . $this->end->ip_int . "'";
		}
		else {
			$sql = "SELECT DISTINCT(host.domain) FROM host WHERE dynamic_range='$this->id'";
		}

		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_object($db_results)) {
			$results[] = new Domain($r->domain);
		}

		return $results;

	} // get_represented_domains

	//get all other ranges in the same subnet as $this (get's $this as well)
	function get_sibling_ranges() {
		global $conf;

		$sql = "SELECT id FROM ranges WHERE subnet='" . $this->subnet . "'";
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_object($db_results)) {
			$results[] = new Range($r->id);
		}

		return $results;

	} //get_sibling_ranges

	//check to see what the status of the zone_range mapping is (enabled/disabled)
	function zone_is_enabled($zone_id) {
		global $conf;

		$sql = "SELECT enabled FROM zone_range WHERE zone='$zone_id' AND range='$this->id'";
		$db_results = mysql_query($sql,$conf['dbh']);

		$results = mysql_fetch_row($db_results);
		
		return $results[0];

	} //zone_is_enabled

	//check to see if $zone_id is in the current range
	function has_zone($zone_id) {
		global $conf;

		if (!$this->zones) { return false; }
		
		foreach ($this->zones as $zone) {
			if ($zone_id == $zone) { return true; }
		} //end foreach

		return false;
	} //has_zone

        /*!
                @function has_domain
                @discussion checks to see if any hosts in $this->id
                            have $domain_id
        */
        function has_domain($domain_id) {

                $hosts = $this->get_range_hosts();

                foreach ($hosts as $host) {
                        if ($host->domain->id == $domain_id) { return true; }
                } // end foreach

                return false;

        } // has_domain

	//return the default domain for $zone_id
	function get_default_domain($zone_id) {
		global $conf;
	
		$sql = "SELECT default_domain FROM zone_range WHERE range='$this->id' AND zone='$zone_id'";
		$db_results = mysql_query($sql, $conf['dbh']);

		$results = mysql_fetch_row($db_results);
		
		return $results[0];

	} //get_default_domain

	// take an int IP and return an IP object
	function get_ip ($ip) {
		global $conf;

		$sql = "SELECT id FROM ip WHERE address='$ip'";
		$db_result = mysql_query($sql, $conf['dbh']);
		$ip_obj = mysql_fetch_object($db_result);
		
		return new IP($ip_obj->id);
	}

	// what if we just want 1 IP?
	function get_free_ip() {
		$ips = $this->get_free_addresses(1);
		return $ips[0];
	}

	/*!
		@function get_free_addresses
		@discussion return an array of IP objects that are available
	*/
	function get_free_addresses($limit = 0) {
		global $conf;

		$start = $this->start->ip_int;
		$end   = $this->end->ip_int;

		//make sure the start and end are inside our ip_block, and it's enabled
		$sql = "SELECT * FROM ip_blocks WHERE start<=$start AND end>=$end AND enabled='1'";
		$db_results = mysql_query($sql, $conf['dbh']);

		$results = mysql_fetch_row($db_results);

		if ($results) {
	
			$found_ip = array();

			//get the list of current used ips
			$sql = "SELECT ip FROM host WHERE ip>=$start AND ip<=$end ORDER BY ip";	
			$db_result = mysql_query($sql, $conf['dbh']);

			while ( $obj = mysql_fetch_object($db_result) ) {
				$ip_int = $obj->ip;
				$found_ip[$ip_int] = "I be found!";
			}	

			$ip_count = $start;

			//recurse through the possible ips and add ones that aren't in use
			do {
				if ( !($found_ip["$ip_count"]) ) { 
					$ips[] = new IP($ip_count); 
				}
				$ip_count = $ip_count + 1;
			} while ($ip_count <= $end);
			

		} //if the start and end enabled and in our ip_blocks

		else {
			return false;
		}

		return $ips;
	} // get_free_addresses


	// just return a count of the available IP addresses for this
	//   range of addresses.
	function get_free_address_count() {
		global $conf;

		$start = $this->start->ip_int;
		$end   = $this->end->ip_int;

		if ($this->type == "static") { 
			$sql = "SELECT count(*) FROM host WHERE ip>=$start AND ip<=$end";
			$db_result = mysql_query($sql, $conf['dbh']);
			$result = mysql_fetch_row($db_result);
			$free_count = $this->total_ips - $result[0];
		}
		elseif ($this->type == "dynamic") { 
			$free_count = $this->total_ips - $this->get_used_ips();
			if ($free_count < 0 ) { $free_count = "<font class=\"error\">Over Filled</font>"; }
		}
		if (!$free_count) { $free_count = "0"; }
		return $free_count;

	} // get_free_address_count


	// return an IP object of a free ip address in this range
	function get_free_ip_address () {
		// get the free ip's but limit to just 1
		$ips = $this->get_free_addresses(1);
		return $ips[0];
	} // get_free_ip_address

	//Delete the $this Range
	function delete_self() {
		global $conf;

		//First Nuke The Zone_range references
		$sql = "DELETE FROM zone_range WHERE range='$this->id'";
		$db_results = mysql_query($sql, dbh());

		//Next Nuke the Range it's self. 
		$sql = "DELETE FROM ranges WHERE id='$this->id'";
		$db_results = mysql_query($sql, dbh());

		//Nuke any object_history
		$sql = "DELETE FROM object_history WHERE type='domain' AND object_id='$this->id'";
		$db_results = mysql_query($sql, dbh());

		// Now add the deletion record
		$text = "Deleted " . $this->type . " range: (" . $this->start->ip_addr . ") -- (" . $this->end->ip_addr . ")";
		$conf['ui']->add_delete_history('0',$text,$user->username);


	} //delete_self


	//check to see if an IP is free or not
	function check_free_ip_address($ip_int) { 
		global $conf;

        	$sql = "SELECT count(*) FROM host WHERE host.ip='$ip_int'";
		$db_results = mysql_query($sql, $conf['dbh']);
		$results = mysql_fetch_object($db_results);		

		//if we find results return false, it's not free!
		if ($results) {
			return false;
		}

		//All else fails return true;
		return true;

	} //check_free_ip_address

	// Change all hosts in this range to be dynamic (for when we change type)
	function make_hosts_dyanmic() {
		global $conf;

		/* First things first, get all the hosts for this range */
		$hosts = $this->get_range_hosts();

		if (!count($hosts)) { return true; }

		//For each host update it!
		foreach($hosts as $host) {
			if ($host->dynamic_range != $this->id) { 
				$host = new Host($host->id);
				$host->update_ip(0,$this->id);
			}
		} //end foreach


	} //make_hosts_dynamic

	// Change all hosts in this range to be static (for when we change the type)
	function make_hosts_static() {

		/* First things first, get all the hosts for this range */
		$hosts = $this->get_range_hosts();

		if (!count($hosts)) { return true; }
		
		// this is tricky, find open ip then assign it 
		foreach($hosts as $host) {
			if (!$this->ip->ip_int) { 
				$ip = $this->get_free_ip();
				$host = new Host($host->id);
				$host->update_ip($ip->ip_int,0);	
			}
		} //end foreach

	} //make_hosts_static

	//update the comment
	function update_zone_comment($comment, $zone_id = 0) {
		global $conf;
	
		if (!$zone_id) { $zone_id = $conf['zone']->id; }

		$sql = "UPDATE zone_range SET comment='$comment' WHERE range='$this->id' AND zone='$zone_id'";
		$db_results = mysql_query($sql, $conf['dbh']);
	
		$this->comment = $comment;

	} //update_comment

	//update the enabled status of this range
	function update_enabled($enabled) {
		
		$this->update_item('enabled',$enabled);
		$this->enabled = $enabled;

	} //update_enabled

	//updates the comment (admin one) 
	function update_comment($comment) {
		
		$this->update_item('comment',$comment);
		$this->comment = $comment;
	
	} //update_comment

	// updates the type of range 
	// this is more complicated because
	// we hae to check for hosts...
	function update_type($type) {

		// We need to modify the hosts if dynamic/ddns
		if (($type == 'dynamic' OR $type == 'ddns') AND ($this->type != 'dynamic' OR $this->type != 'ddns') ) {
			$this->make_hosts_dyanmic();
		}
		// If static
		elseif ($type == 'static' AND $this->type != 'static') {
			$this->make_hosts_static();
		}

		$this->update_item('type',$type);
		$this->type = $type;
	
	} //update_type

	//updates the end ip of the range
	function update_end($ip_end) {

		$this->update_item('end',$ip_end);
		$this->end = new IP($ip_end);

	} //update end

	//updates the start ip of the range
	function update_start($ip_start) {
		
		$this->update_item('start',$ip_start);
		$this->start = new IP ($ip_start);

	} //update_start

	function update_allow_all_hosts($allow) {
		
		$this->update_item('allow_all_hosts',$allow);
		$this->allow_all_hosts = $allow;

	} //update_start

	//updates the default subnet for $this
	function update_subnet($subnet) {
		
		$this->update_item('subnet',$subnet);
		$this->subnet = $subnet;

	} //update_subnet

	//update item, used for other updates
	function update_item($type,$new_item) {
		global $conf;

		$sql = "UPDATE ranges SET $type='$new_item' WHERE id='$this->id'";
		$db_results = mysql_query($sql, $conf['dbh']);

	} //update_item

	//add $zone_id to this range (update zone_range mapping)
	function add_zone($zone_id,$default_domain) {
		global $conf;

		$sql = "INSERT INTO zone_range (zone,range,default_domain,enabled) VALUES ('$zone_id','$this->id','$default_domain','1')";
		$db_results = mysql_query($sql, $conf['dbh']);


	} //add_zone

	//delete the zone_range mapping for this zone and $zone_id
	function delete_zone_range($zone_id) {
		global $conf;

		$sql = "DELETE FROM zone_range WHERE range='$this->id' AND zone='$zone_id'";
		$db_results = mysql_query($sql, $conf['dbh']);

		$zone = new Zone($zone_id);

		// Add a deletion record
		$text = "Deleted Zone --> Range Mapping (" . $this->start->ip_addr . ") -- (" . $this->end->ip_addr . ") to " . $zone->name;
		$conf['ui']->add_delete_history($zone_id,$text,$user->username);

	} //delete_zone_range

	//moves all zone_range references from one zone to another
	function replace_zone_range($range_id,$merge_range) {
		global $conf;

		//First get a list (so we don't duplicate!)
		$sql = "SELECT * FROM zone_range WHERE range=$range_id OR range=$merge_range";
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_object($db_results)) {
			$zone_ranges[$r->zone] = $r;
		}

		print_r($zone_ranges);

		//Now Nuke all the range references
		$sql = "DELETE FROM zone_range WHERE range='$merge_range' OR range='$range_id'";
		$db_results = mysql_query($sql, $conf['dbh']);


		//Now use a Insert and update for new (so we don't duplicate)
		foreach ($zone_ranges as $zone_range) {

			$sql = "INSERT INTO zone_range (zone,range,default_domain,comment,enabled) VALUES ('$zone_range->zone','$range_id','$zone_range->default_domain','$zone_range->comment ','$zone_range->enabled')";
			$db_results = mysql_query($sql, $conf['dbh']) or die ("Invalid query: ". mysql_error());

		}


	} //replace_zone_range

	//append the set range to $this
	function append_range($range_id) {
		global $conf,$user;

		//first get the information about the range we are going to append
		$new_range = new Range($range_id);
		$end = $new_range->end->ip_int;

		//update the current range with the passed range's end
		$sql = "UPDATE ranges SET end='$end' WHERE id='$this->id'";
		$db_results = mysql_query($sql, dbh());

		//delete the old range..
		$sql = "DELETE FROM ranges WHERE id='$range_id'";
		$db_results = mysql_query($sql, dbh());

		// add record of deleting the old range
		$text = "Deleted Old range when appending RangeID: $range_id";
		$conf['ui']->add_delete_history(0,$text,$user->username);

	} //append_range

	//prepend the set range to $this
	function prepend_range($range_id) {
		global $conf,$user;

                //first get the information about the range we are going to append
                $new_range = new Range($range_id);
		$start = $new_range->start->ip_int;

                //update the current range with the passed range's end  
                $sql = "UPDATE ranges SET start='$start' WHERE id='$this->id'";
                $db_results = mysql_query($sql, dbh());

                //delete the old range..
                $sql = "DELETE FROM ranges WHERE id='$range_id'";
                $db_results = mysql_query($sql, dbh());

		// add record of the deleted range
		$text = "Delete Old range when prepending RangeID:$range_id";
		$conf['ui']->add_delete_history(0,$text,$user->username);

        } //prepend_range

	//remove the workgroup entry for the specified range
	function remove_workgroup_range($range_id) {
		global $conf;

		$sql = "DELETE FROM workgroup_range WHERE range=$range_id";
		$db_results = mysql_query($sql, $conf['dbh']);

	} //remove_workgroup_range

	/*!
		@function update_default_domain
		@discussion Updates the default domain for said range
	*/
	function update_default_domain($zone_id,$domain_id) {
		global $conf;

		$sql = "UPDATE zone_range SET default_domain='$domain_id' WHERE zone='$zone_id' AND default_domain=$this->id";
		$db_results = mysql_query($sql, $conf['dbh']);

		if ($db_results) { return true; }
		
		return false;

	} // update_default_domain

	//update host_range if it's a dynamic host
	function replace_dynamic_range($range_id,$merge_id) { 
		global $conf;

		$sql = "UPDATE host SET dynamic_range=$range_id WHERE dynamic_range=$merge_id";
		$db_results = mysql_query($sql, $conf['dbh']);

	} //update_host_range

	//create a select of the two possible ranges that they can merge
	function show_merge_range_select($ip_start,$ip_end,$range_type) {
		global $conf;

		$sql = "SELECT ranges.* FROM ranges WHERE ranges.type='$range_type' AND (ranges.start=$ip_end+1 OR ranges.end=$ip_start-1)";
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_object($db_results)) {
			$range = new Range($r->id);
			$ranges[] = $range;		
			}
		
		//if we find at least one then...
		if (count($ranges) > 1) {
			echo "<select name=\"merge_range\">";
			foreach ($ranges as $range) {
				echo "<option value=\"$range->id\">". $range->start->ip_addr . "&nbsp;--&nbsp;" . $range->end->ip_addr . "</option>";
				}
			echo "</select>";
		}
		elseif (count($ranges) == '1') {
			echo "<input type=\"hidden\" name=\"merge_range\" value=\"". $ranges['0']->id ."\">\n";
			echo $range->start->ip_addr . "&nbsp;--&nbsp;" . $range->end->ip_addr . "<br />";	

		} 
		//if not ohh well better luck next time
		else {
			echo "There are no ranges that can be merged.";
		}

	return $ranges;

	} //show_merge_range_select

	//grabs the workgroup_range info from one range and duplicates it in the new one
	//used for spliting ranges
	function duplicate_workgroup_range($src_range,$dst_range) { 
		global $conf;

		$sql = "SELECT * FROM workgroup_range WHERE range='$src_range'";
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($w = mysql_fetch_object($db_results)) {
			$workgroups[] = $w;
		} //end while

		if ($workgroups) {

		foreach ($workgroups as $workgroup) { 
			$sql = "INSERT INTO workgroup_range (workgroup,range) VALUES ($workgroup->workgroup,$dst_range)";
			$db_results = mysql_query($sql, $conf['dbh']);
			} //end foreach

		} //end check for $workgroups

	} //duplicate_workgroup_range

	//get the hosts that are in this range....
	function get_range_hosts() {
		global $conf;
		
		//if it's a static address use this sql statement
		if ($this->type == "static") {
			$start = $this->start->ip_int;
			$end = $this->end->ip_int;
			$sql = "SELECT host.id FROM host WHERE ip>='$start' AND ip<='$end' ORDER BY ip DESC";
		} //end if for static
		elseif ($this->type == "dynamic") {
			$sql = "SELECT host.id FROM host WHERE dynamic_range='$this->id'";
		} //end elseif for dynamic

		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_row($db_results)) {
			$host = new Host($r[0]);
			$hosts[] = $host;
		} //end while

		return $hosts;
	
	} //get_range_hosts

	//counts the number of hosts in a specific range
	function get_used_ips() {
		global $conf;
	
                $start = $this->start->ip_int;
                $end   = $this->end->ip_int;

		//if it's dynamic 
		if ($this->type == "dynamic") {
        	       $sql = "SELECT count(*) FROM host WHERE dynamic_range='$this->id'";
		} 
		//if it's static
		else { 
	               $sql = "SELECT count(*) FROM host WHERE ip>=$start AND ip<=$end";
		}
                $db_result = mysql_query($sql, $conf['dbh']);
                $result = mysql_fetch_row($db_result);

                return $result[0];
	
		
	} //used_ip_count

	function is_enabled() 
	{
		return $this->enabled;
	}

}

?>
