<?php
/*

 $Id: ui.php,v 1.181 2004/05/12 21:20:55 philips Exp $
 $Source: /data/cvsroot/maintain/modules/class/ui.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*!
	@header User Inferface Class
	The base UI class for showing things like hosts, domains, etc.
	Create this object and use it in any of the pages in docs/

*/

class UI {

	var $widget;

	function UI ( ) {
		
		$this->widget = new Widget();

	} // UI

	// a plain text formatted text box.
	function show_text_box ($title, $text, $width) {
		global $conf, $user;
		
	        require $conf['prefix'] . "/templates/text.inc";
	} // show_text_box

	/**
	*
	* Shows a select pulldown
	* 
	* @param	$name		name of pulldown menu, both id and name
	* @param	$objects	an array of objects $object->id used as value
	* @param	$current	the default option
	* @access	public
	*/
	function show_select($name, $objects, $current) {
	
		echo "<select id=\"$name\" name=\"$name\">\n";

		foreach ($objects as $key => $value) { 

			if (is_object($value)) { $value = $value->id; }

			if (strcasecmp($value,$current) == 0) { 
				echo "\t<option value=\"$value\" SELECTED>$key</option>\n";
			} // end if current
			else {
				echo "\t<option value=\"$value\">$key</option>\n";
			} // end else

		} // end foreach

		echo "</select>\n";

	} // show_select

	//alternate the bgcolor of our standard tables using
	//dark_bgcolor and light_bgcolor
	function alternate_bgcolor() {
		global $conf;

		if ($conf['current_table_color'] == $conf['color']->bg['body_light']) {
			$conf['current_table_color'] = $conf['color']->bg['body_dark'];
		}
		else {
			$conf['current_table_color'] = $conf['color']->bg['body_light'];
		}
	
	echo $conf['current_table_color'];
		
	}

	/*!
		@function flip_class() {
		@discussion takes two classes and flips them back and forth
			used for the alternating colors on the rows. The classes
			are passed in at the start, and can be clobbered by setting clobber=1
	*/
	function flip_class($class_array=0,$clobber=0) {
		static $classes = array();
		
		if ($clobber) {
			$classes = $class_array;
		}
		else {
			
			$classes = array_reverse($classes);
			return $classes[0]; 
		}
		
	} // flip_class

	/*!
		@function show_arrow
		@discussion shows the revelent arrow (desc/asc) based off of the user view
	*/
	function show_arrow() {
		global $conf,$user;

		if (! strcasecmp($user->view->sort_order, "desc") ) {
			echo "<img src=\"". $conf['web_path'] . "/images/desc_order.gif\" title=\"Decending Order\" />";
		}
		else {
			echo "<img src=\"". $conf['web_path'] . "/images/asc_order.gif\" title=\"Decending Order\" />";
		}
	
	} // show_arrow

	/*!
		@function show_status
		@discussion takes a 0/1 boolean value and returns Enabled or Disable
	*/
	function show_status($value) {

		if ($value) { 
			return "Enabled";
		}
		else {
			return "Disabled";
		}

	} // show_status

	/**
	*
	* Creates a string that tells when a object will be next updated.
	*
	* This functions tests for all cases and will return one of three cases:
	* - Host up to date: "$name is up to date"
	* - Host will update: "This $name will be updated in $minutes_remaining
	*						minutes on $date at $time"
	* - Updates have stopped: "Update have been paused."
	*
	*
	* @return	string	 Description
	* @param	$last_update 	The last time this object was updated 
	*							@see ObjectHistory
	* @param	$name			The type of object being updated (ex. host)
	* @param	$domain_id		The ID of the domain that this object belongs
	*							too, used to find the refresh time, if this is
	*							not passed in the function uses 
	*							$conf['update_time']
	* @access	public
	*/
	function show_live_time($last_update, $name, $domain_id = 0) {
		global $conf;

		$update_interval = $conf['update_time']; //default to config file setting
        
		if ($domain_id) {
        	// If a domain_id was passed try to get the SOA refresh time to use
			// instead of $conf['update_time'] 
        	$sql = "SELECT soa.refresh FROM soa,domain,domain as master_domain where ( domain.master_domain=master_domain.id AND soa.domain=master_domain.id AND domain.id='$domain_id') OR soa.domain='$domain_id' LIMIT 1";
        	$db_results = mysql_query($sql, dbh());
			$results = mysql_fetch_object($db_results);

			if ($results->refresh) { $update_interval = $results->refresh; }
		}
		
		// Double the estimate so customers don't complain
		$estimated_time = $last_update + ($update_interval * 2); 

		// Remaining number of minutes is	
		$minutes_remaining = date('i', $estimated_time - time());
		// Estimated date 
		$date = date('l F d Y', $estimated_time);	
		// Estimated time
		$time = date('h:i a', $estimated_time);

		if ($estimated_time <= time()) {
			return "<strong>$name is up to date</strong>";
		}

		$stop_file = $conf['prefix'] . "/build/stop.update";

		if (@file_exists($stop_file)) { 
			return "<font class=\"error\">Updates have been paused. Please wait 10 minutes and try again. </font>";
		}

		
		if ($minutes_remaining > 1) { 
			$text = "<font class=\"error\">This $name will be updated in $minutes_remaining minutes on $date at $time.</font>";
		}
		else {
			$text = "<font class=\"error\">This $name will be updated in less than a minute.</font>";
		}

		return $text;

	} // show_live_time
	
	/*!
		@function show_access_denied
		@discussion Give them big red text if they have tried to go somewhere they should be able to
	*/
	function show_access_denied() {
		global $conf;

		echo "<p class=\"fatalerror\">Error Access Denied</p>\n";
		echo "<a href=\"" . $conf['web_path'] . "\">[ Return ]</a>";
		show_template('footer');
		exit();
		
	} // show_access_denied

	// given a host object, show its information (kind of a verbose
	//   view of the host)
	function show_host ($host, $action_type=0) {
		global $conf, $user;

	        require $conf['prefix'] . "/templates/show_host.inc";
	}

	// given a ip_int,ha,host_id show the host_history
	// information
	function show_host_history($host_id=0,$ip_int=0,$ha=0) {
		global $conf;

		require $conf['prefix'] . "/templates/show_host_history.inc";

	} //show_host_history

	/*!
		@function show_zone_history
		@discussion displays the update/deletion history template 
			for the passed zone
	*/
	function show_zone_history($zone_id) {
		global $conf;

		require ($conf['prefix'] . "/templates/show_zone_history.inc");

	} // show_zone_history

	function show_domain($domain, $action_type=0) {
		global $conf, $user;
		
		require $conf['prefix'] . "/templates/show_domain.inc";
	}
	
	// given a cname show the information
	function show_cname ($cname, $action_type = 0) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_cname.inc";
		
	}

	// given a mx show the information

	function show_mx ($mx, $action_type = 0) 
	{

		global $conf, $user;
		
		require $conf['prefix'] . "/templates/show_mx.inc";
	
	} // show_mx

	function show_zone($zone, $action_type = 0)
	{
		global $conf, $user;
		
		require $conf['prefix'] . "/templates/show_zone.inc";
	
	} // show_zone

	// given an array of host objects recurse through them and
	//   display them in a purdy table
	function show_hosts ( $hosts, $total_items = 0 ) {
		global $conf, $user;

		if ( ! $total_items ) {
			$total_items = count_host($conf['zone']->id);
		}
	        require $conf['prefix'] . "/templates/show_hosts.inc";
	}

	function show_zones ( $zones, $total_items = 0) {
		global $conf, $user;
		
		require $conf['prefix'] . "/templates/show_zones.inc";
	}
	
	function show_domains($domains, $total_items = 0) {
		global $conf, $user;
		require $conf['prefix'] . "/templates/show_domains.inc";
	}

	// cheeseball fix for IE crappieness
	function ie_footer_fix() {
		echo "<br /><br /><br /><br /><br /><br /><br /><br />\n";
		echo "<br /><br /><br /><br /><br /><br /><br /><br />\n";
	} //ie_footer_fix

	/*!
		@function add_delete_history
		@discussion this function adds a object_history entry 
			recording the deletion of an object
	*/
	function add_delete_history($zone_id,$comment,$username) {

		$time = time();

		$sql = "INSERT INTO object_history (username,object_type,object_id,date,comment) VALUES" . 
			" ('$username','delete','$zone_id','$time','$comment')";
		$db_results = mysql_query($sql, dbh());

	} // add_delete_history


	/*!
		@function parse_dhcp_option
		@discussion takes a dhcp option and returns a version of it
			with the %h replaced for the hostname
	*/
	function parse_dhcp_option($value,$object) {
		
		// Must be a host!
		if ($object->type == 'host') { 
			$value = str_replace("%h","$object->hostname",$value);
			$value = str_replace("%i","$object->ip->ip_addr",$value);
			$value = str_replace("%m","$object->ha",$value);
			$value = str_replace("%6m",substr($object->ha,0,6),$value);
		}
		else {
			$value = str_replace("%h","[ Hostname ]",$value);
			$value = str_replace("%i","[ IP Address ]",$value);
			$value = str_replace("%m","[ Hardware Address ]",$value);
			$value = str_replace("%6m","[ First 6 Hardware Address ]",$value);
		} 			


	return $value;

	} // parse_dhcp_option


	/*!
		@function show_unregistered_hosts
		@discussion shows the template for displaying unregistered hosts
		@param $hosts	Hosts we are displaying
		@param $date	Date of the data
	*/
	function show_unregistered_hosts($hosts,$date,$day_count=0,$next_action) {
		global $conf;

		require ($conf['prefix'] . "/templates/show_unregistered.inc");

	} // show_unregistered_hosts

	/*
		@function show_confused_hosts
		@discussion Shows hosts that are registered, but are using the wrong IP
		@param $hosts	The Hosts we want to show
		@param $date	Date of the data we want to show
	*/
	function show_confused_hosts($hosts,$date,$day_count=0,$next_action) {
		global $conf;

		require ($conf['prefix'] . "/templates/show_unregistered.inc");

	} // show_confused_hosts

	/*!
		@function clean_expire_date
		@discussion cleans up the expired date removing non-numeric values then converts it to a unix time stamp
	*/
	function clean_expire_date($date) {
			return $strtotime($date);
	} // clean_expire_date

	/*!
		@function clean_hardware_address
		@discussion Cleans any non-valid chr's from a hardware address, and make it lowercase. Returns the cleaned 
			    hardware address.
		@param $mac	The hardware address
	*/
	function clean_hardware_address($mac) {

	        // let's remove any invalid chr's from their mac addie
                $mac = preg_replace("/[G-Zg-z]+|\-|\.|\s+|\_|:/", "", $mac);


		// make it lowercase for the sake of apperance, and for radius..
		$mac = strtolower($mac);

	return $mac;

	} //clean_hardware_address

	/*!
		@function is_hardware_address
		@discussion checks to make sure it's a valid hardware address, strips invalid chrs first
		@param $mac	The Hardware address we are checking.
	*/
	function is_hardware_address($mac) {
		global $conf;
	
		// first clean it up!
		$mac = $this->clean_hardware_address($mac);

                // is it 12 chr's long
                if ( strlen($mac) != 12 ) {
			return false;
                }

                // does it have valid chrs
                if ( preg_match("/[^0-9a-fA-F]+/", $mac) ) {
			return false;
                }

		return $mac;
	
	} // is_hardware_address

	/*!
		@function is_domain
		@discussion returns true if it is determined to be a 
			domain name ( currently 1st level domains are hardcoded in here )
	*/
	function is_domain($domain_name) {
		
		$first_lv_domains = array ('org','edu','com','net','biz','info','mil','gov','name','coop','aero','int','museum','pro');

		if (! ( preg_match("/^[\w\-]+\.[\w\-]+/",$domain_name) || preg_match("/^[\w\-\.]{2,}$/",$domain_name) ) 
			&& ! ( in_array($domain_name,$first_lv_domains) && preg_match("/^[A-Za-z]{2}$/",$domain_name) ) ) {

			return false;

		}

		return true;

	} // is_domain

	/*!
		@function mysql_auth
		@discussion returns true if they are doing
			    mysql_auth
	*/
	function mysql_auth() {
		
		$auths = libglue_param('auth_methods');

		if (is_array($auths)) { 
			foreach ($auths as $auth) { 
				if ($auth == 'mysql') {
					return true;
				}
			} // end foreach
		} 
		elseif ($auths == 'mysql') { return true; }
		return false;

	} // mysql_auth

	/*!
		@function ldap_auth
		@discussin returns true if we are running ldap
			   auth false if not.
	*/
	function ldap_auth() {
		
		$auths = libglue_param('auth_methods');

		if (is_array($auths)) {
			foreach ($auths as $auth) {
				if ($auth = 'ldap') {
					return true;
				}
			} // end foreach
		} 
		elseif ($auths == 'ldap') { return true; }
	
		return false;

	} // ldap_auth

	// DEAD CODE DEAD CODE //
	/*!
		@function orst_domains
		@discussion get any orst.edu domains... this is so we can figure out if they have any
			    domains that need to be migrated still
	*/
	function orst_domains($all_domains) {
		global $conf;
	
		if ($all_domains) {
			foreach ($all_domains as $domain) {
	        		if ( preg_match("/orst.edu$/", $domain->name) ) {
		                	$zone_id = $conf['zone']->id;
	
	                        	$sql = "SELECT count(*) FROM host WHERE zone='$zone_id'".
        	        	                " AND domain='$domain->id'";
	        	                $db_result = mysql_query($sql, $conf['dbh']);
	
	        	                $count = mysql_fetch_row($db_result);
	
		                        // woohoo!!  They have some domains!
		                        if ( $count[0] ) {
		                                $domains[] = $domain;
		                        } //add domain to array
	        	        } //enc preg check
		        } //end foreach
		} //end check to make sure there ARE domains

	return $domains;


	} // orst_domains

	/*!
		@function show_dhcp_space_select
		@discussion This get's the distinct space types from 
			dhcp_class. 
	*/
	function show_dhcp_space_select($spaces=0,$name=0) {

		if (!$spaces) { $this->get_dhcp_space("SELECT distinct(space) FROM dhcp_class"); }
		if (!$name) { $name = "space"; }

		// If we still don't have anything
		if (!is_array($spaces)) { 
			echo "<input type=\"hidden\" id=\"space\" value=\"false\" />";
			echo "<div class=\"error\">No Vender Space Found: Define a DHCP Class first</div>";
			return true;
		}

		echo "<select name=\"$name\" disabled>\n";

		foreach ($spaces as $data) {

			echo "\t<option value=\"" . $data->space . "\">" . ucasefirst($data->space) . "</option>\n";

		} // end foreach spaces

		echo "</select>\n";

		return true;

	} // show_dhcp_space_select

	/*!
		@function show_bandwidth_rate_select
		@discussion This shows the different possible bandwidth_rate values as a select
	*/
	function show_bandwidth_rate_select($bandwidth_rate) {
		global $conf;

		echo "<select name=\"bandwidth_rate\">\n";
		echo "\t<option value=\"full\""; if ($bandwidth_rate == "full") { echo "SELECTED"; } echo " >Normal/Full</option>\n";
		echo "\t<option value=\"limited\""; if ($bandwidth_rate == "limited") { echo "SELECTED"; } echo " >Temporarily Limited</option>\n";
		echo "\t<option value=\"blocked\""; if ($bandwidth_rate == "blocked") { echo "SELECTED"; } echo " >Limited</option>\n";
		echo "</select>";

	} //show_bandwidth_rate_select
	
	// show the unregistered day count select
	function show_unregistered_day_select($day_count=0) {

		$days = array("1","2","3","4","5","10");
		
		echo "<select name=\"day_count\">\n";

		foreach ($days as $day) {
			echo "\t<option value=\"$day\"";
			if ($day == $day_count) { echo " SELECTED "; }
			echo ">$day</option>\n";
		}

		echo "</select>\n";

	} //show_unregistered_day_select

	// show the unregistered date possibilities
	function show_unregistered_host_select($date=0) { 
		global $conf;

		echo "<select name=\"date\">\n";

		$sql = "SELECT DISTINCT(date) FROM host_history";
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_object($db_results)) {

			if (date("G",$r->date) == "0" AND date("i",$r->date) == "30" ) { 
				echo "<option value=\"$r->date\"";
				if ($date == $r->date) { echo " SELECTED "; }
				echo ">" . date("F j, Y g:i a", $r->date) . "</option>\n";
			}

		} //end while

		echo "</select>";


	} //show_unregistered_host_select

	//this is for the ranges, show any subnets that fit inside.
	function show_range_subnet_select($range_start,$range_subnet,$name=0) {
		global $conf;

		$sql = "SELECT id FROM subnet WHERE subnet<='$range_start' ORDER BY subnet DESC";
		$db_results = mysql_query($sql, $conf['dbh']);

		if (!$name) { $name="subnet_id"; }

		echo "<select name=\"$name\">\n";
		while ($r = mysql_fetch_object($db_results)) {
			$subnet = new Subnet($r->id);
			echo "<option value=\"$subnet->id\"";
			if ($range_subnet == $r->id) { echo " SELECTED "; }
			echo ">$subnet->name Subnet(". $subnet->subnet->ip_addr .") Netmask(". $subnet->netmask->ip_addr . ")</option>\n";
		}
		echo "</select>";



	} //show_range_subnet_select

	//Show possible subnet netmask values as a select
	function show_netmask_select($netmask_int,$name=0) {
		global $conf;
		
		if (!$name) { $name = 'netmask[]'; } 
		$netmasks = array ('255','254','252','248','240','224','192','128','0');
	
		echo "<select name=\"$name\">\n";
		foreach ($netmasks as $netmask) {
			echo "\t<option value=\"$netmask\" ";
			if ($netmask == $netmask_int) { echo "SELECTED"; }
			echo " >$netmask</option>\n";
		}
		echo "</select>\n";

	} //show_netmask_select

	// This shows the zones based upon an array of zone objects
	function show_zone_select ( $zones, $selected = 0 ,$name = 0) {
	
		//if we don't set the name then it's zone_id
		if (!($name)) { $name="zone_id"; }

		if (count($zones) >= "2") {
			print(" <select name=\"$name\">\n");

			foreach ($zones as $zone) {
				if ( $selected == $zone->id ) {
					print("         <option value=\"$zone->id\" SELECTED>$zone->name</option>\n");
				}
				else {
					print("         <option value=\"$zone->id\">$zone->name</option>\n");
				}
			}

			print(" </select>\n");
		}
		elseif (count($zones) == "0") {
			echo "No Zones";
		}
		elseif (count($zones) == "1") {
			$zone = new Zone($zones[0]);
			echo $zone->name;
		} 
	
	} //show_zone_select

	//display domains in a select (depends upon a array of domain objects)
	function show_domain_select($domains, $selected =0,$name =0,$disabled='') 
	{
		if (!$name) { $name = "domain_id"; }
		echo "<select $disabled id=\"$name\" name=\"$name\">\n";

		//if they hacvn't chosen one then set it to the first one on the list
		if (!$selected) { $selected = $domains[0]->id;  }

		foreach  ($domains as $d) {
			if ($selected == $d->id) {
			print ("<option value=\"$d->id\" SELECTED>$d->name</option>\n");
			}
			else {
			print ("<option value=\"$d->id\">$d->name</option>\n");
			}
		}
		print (" </select>\n");
	} //show_domain_select

	//displays a select for workgroups depends upon a array of workgroup objects
	function show_workgroup_select($workgroups, $selected = 0) {
		print "<select name=\"workgroup_id\">\n";
	
		foreach  ($workgroups as $w) {
			if ($selected == $w->id) {
				print ("<option value=\"$w->id\" SELECTED>$w->name</option>\n");
			}
			else {
				print ("<option value=\"$w->id\">$w->name</option>\n");
			}
		}
		print (" </select>\n");
	} // show_workgroup_select

	
	//displays a select for workgroups depends upon a array of workgroup objects
	/*!
		@function show_enabled_select
		@discussion shows a enabled/disabled select
 	*/
	function show_enabled_select($selected = 1,$name=0) {

		if (!$name) { $name = 'enabled'; }
	
		echo "<select name=\"$name\">\n";
	
		if ($selected) {
			$select_enabled = 'SELECTED';
		}
		else {
			$select_disabled = 'SELECTED';
		}

		print ("<option value=\"1\" $select_enabled>Enabled</option>\n");
		print ("<option value=\"0\" $select_disabled>Disabled</option>\n");

		print (" </select>\n");

	} // show_enabled_select


	//displays a select of the current users. Depends upon an array of objects.
	function show_user_select($users, $selected = 0) {
		global $user, $conf;

		echo "<select name=\"user_id\">\n";
		if (!$selected) { echo "<option value=\"0\" SELECTED>Please Choose a User</option>\n"; }
		foreach ($users as $u) {
			if ($selected == $u) {
				print ("<option value=\"$u->id\" SELECTED>$u->username</option>\n");
			}
			else {
			print ("<option value=\"$u->id\">$u->username</option>");
			}
		}
		print ("\n</select>\n");

	} //show_user_select
		

	/*!
		@function show_list_header
		@discussion show the list header that shows 1,2,3, etc for a list of items
		@param $total_items	Total number of items to show a list for
		@param $user-&gt;view	The view object... depends on this
	*/
	function show_list_header($total_items) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/list_header.inc";

	} // show_list_header

	/*!
		@function show_ranges
		@discussion given an array of ranges display them
		@param $ranges	Array() of range objects
	*/
	function show_ranges($ranges,$total_items) {
		global $conf, $user;
				
		require $conf['prefix'] . "/templates/show_ranges.inc";

	} //show_ranges

	// given a host and IP ranges show the IP ranges and give the ability
	//   to change the IP.
	function show_ip_ranges($host, $ranges) {
		global $conf, $user;

	        require $conf['prefix'] . "/templates/show_ranges.inc";
	} // show_ip_ranges


	// $range is a Range object
	function show_range($action, $range) {
		global $conf, $user;

	        require $conf['prefix'] . "/templates/show_range.inc";
	} // show_range

	// given an array of Range objects create a pulldown with $selected
	//   chosen.
	function show_range_select ( $ranges, $selected = 0 ) {
		global $conf;

		// we don't want to create _all_ of these zone objects because its
		//    totally redundant ... all we really want are zone names.  So
		//    we pre-build this array with id -> name mapping for the zone
 		//    and can save ourselves _a lot_ of select statements.
		$domains = array();

		$sql = "SELECT * FROM zone";
		$db_result = mysql_query($sql, $conf[dbh]);

		while ( $obj = mysql_fetch_object($db_result) ) {
			$domains[$obj->id] = $obj->name;
		}

		print "<select name=\"range_id\">\n";
	
		if (!$selected) { echo "<option value=\"0\" SELECTED>Please Choose a Range</option>\n"; }

		foreach  ($ranges as $r) {
			$zone  = $domains[$r->zone];
			$start = $r->start->ip_addr;
			$end   = $r->end->ip_addr;

			$text = "$zone ($start to $end)";

			if ($selected == $r->id) {
				print ("<option value=\"$r->id\" SELECTED>$text</option>\n");
			}
			else {
				print ("<option value=\"$r->id\">$text</option>\n");
			}
		}
		print (" </select>\n");	
	}

	function show_split_range ($range) {
		global $conf, $user;

	        require $conf['prefix'] . "/templates/show_split_range.inc";
	}


	// $ranges is an array of start, end and netmask data
	function show_unused_ranges($ranges) {
		global $conf, $user;

	        require $conf['prefix'] . "/templates/show_unused_ranges.inc";
	} // show_unused_ranges


	function show_ips ($host, $ips) {
		global $conf, $user;
		if ($ips) {
		        require $conf['prefix'] . "/templates/show_ips.inc";
		}
		else {
			echo "<p align=\"center\"><font class=\"fatalerror\">No IPs Found</font></p>\n";
		}
	}

	function show_confirm($request_uri, $question) {
		print("<p><form name=\"confirm\" method=\"post\" enctype=\"multipart/form-data\" style=\"Display:inline\" action=\"$request_uri\">\n");
		print("$question<br /><br />\n");
		print("<input type=\"submit\" name=\"act\" value=\"Confirm\">\n");
		print("</form>\n");
		print("<form name=\"cancel\" method=\"post\" enctype=\"multipart/form-data\" style=\"Display:inline\" action=\"" . $conf['web_path'] . "/index.php\">\n");
		print("<input type=\"submit\" name=\"action\" value=\"Cancel\">\n");
		print("</form></p>\n");
	}

	// show the given users
	function show_users ($users) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_users.inc";
	}

	/*!
		@function show_local_users
		@discussion shows local mysql users 
	*/
	function show_local_users($users) {
		global $conf, $user;
		
		if (count($users)) { 
			require $conf['prefix'] . "/templates/show_local_users.inc";
		}
		else {
			echo "<p align=\"center\" class=\"fatalerror\">No Users Found</p>";
		}

	} // show_local_users

	function show_add_user () {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_add_zone_user.inc";
	}

	// display the mx records
	function show_mxs($mxs, $total_items)
	{
		global $user, $conf;

		require $conf['prefix'] . "/templates/show_mxs.inc";

	} //show_mxs


	// display the cnames
	function show_cnames($cnames, $total_items)
	{
		global $user, $conf;
		require $conf['prefix'] . "/templates/show_cnames.inc";
	} // show_names

	// show one vlan
	function show_vlan ($action, $vlan) {
		global $user, $conf;

		require $conf['prefix'] . "/templates/show_vlan.inc";
	} // show_vlan

	function show_vlans ($vlans, $total_items) {
		global $user, $conf;
		require $conf['prefix'] . "/templates/show_vlans.inc";
	} // show_vlans


	// show one staging user 
	function show_staging ($action, $staging) {
		global $user, $conf;

		require $conf['prefix'] . "/templates/show_staging.inc";
	} // show_staging


	function show_stagings ($action, $stagings) {
		global $user, $conf;
		require $conf['prefix'] . "/templates/show_stagings.inc";
	} // show_stagings


	function display_field($field) {
		global $conf,$user;

		if ($field <= "") { $field = "&nbsp;"; }
		echo $field; 
		
	}

	//for use whenever something requires a confirmation
	//such as mail being sent or changes being made
	function display_confirmation($title, $message, $nexturl)
	{

		global $conf, $user;

		//it's so small let's just put it here!
		echo "<p><font class=\"header2\">$title</font></p>\n";
		echo "<p>$message</p>\n";
		echo "<p><a href=\"$nexturl\">[Continue]</a></p>";


	} //display_confirmation
		
	//sends e-mail, always from support_email_from takes subject messasge and to, if no to send then support_email used
	function send_mail($subject, $mail_message, $to=0)
	{
		global $conf, $user;
	
		//setup the vars
		$real_from = stripslashes($conf['support_email_from']);
		$mail_message .= "\n\n Sent by $user->username , $user->fullname";
		$url = "http://" . $_SERVER["SERVER_NAME"] . $_SERVER["REQUEST_URI"];
		if (!$to) { $to = stripslashes($conf['support_email']); }
		
		//send the mail
		$sent = mail ("$to","$subject",$mail_message,
			"From: $real_from\r\n" .
			"X-Mailer: Net Mail Gateway PHP/" . phpversion() . "\r\n" .
			"X-Mailer-URL: $url\r\n" .
			"X-Originating-IP: $REMOTE_ADDR"); 

		if ($sent) { return true; } 
		else { return false; }
	} //send_mail

	/*!
		@function get_ranges
		@discussion get's ranges based on $user->view or passed sql statement, returns range objects
		@param $sql	Passed SQL statement
	*/
	function get_ranges($sql = 0) {
		global $conf, $user;

		$ranges = array();
		
		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, $conf['dbh']);
		
		while ($r = mysql_fetch_object($db_results)) {
			$range = new Range($r->id);
			$ranges[] = $range;
		}
	
	return $ranges;

	} //get_ranges

	/*!
		@function get_dhcp_space
		@discussion gets the dhcp spaces
	*/
	function get_dhcp_space($sql = 0) {
		global $user;
		
		$spaces = array(); 

		if (!$sql) { $sql = $user->get_query(); }
		$db_results = mysql_query($sql, dbh()); 

		while ($r = mysql_fetch_object($db_results)) {
			$spaces[] = $r;
		}

		return $spaces;
	
	} // get_dhcp_space

	//get an array of workgroups
	function get_workgroups($sql = 0) {
		global $conf, $user;

		$workgroups = array();
		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, $conf['dbh']);
		
		while ($r = mysql_fetch_object($db_results)) {
			$workgroup = new Workgroup($r->id);
			$workgroups[$workgroup->name] = $workgroup;
		}

	return $workgroups;

	} //get_workgroups

	//get an array of hosts
	function get_hosts($sql = 0) {
		global $conf, $user;
	
		$hosts = array();
		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, $conf['dbh']);
		while ($r = mysql_fetch_object($db_results)) {
			$host = new Host($r->id);
			$hosts[] = $host;
		}

	return $hosts;

	} //get_hosts

	//get an array of domains
	function get_domains($sql = 0)
	{
		global $conf, $user;

		$domains = array();
		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, $conf['dbh']);
		while ($d = mysql_fetch_object($db_results))
			{
			$domain = new Domain($d->id);
			$domains[] = $domain;
			}
		return $domains;

	} //get_domains

	// get and array of the cnames and setup the view
	function get_cnames($sql = 0) 
	{
		global $conf, $user;
		
		$cnames = array();
		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($c = mysql_fetch_object($db_results)) {
			$cname = new Cname($c->id);
			$cnames[] = $cname;	
		}

		return $cnames;
	} // get_cnames
		
	// get an array of the zones
	function get_zones($sql = 0)
	{
		global $conf, $user;
		
		$zones = array();
		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, $conf['dbh']);
		while ($z = mysql_fetch_object($db_results)) {
			$zone = new Zone($z->id);
			$zones[] = $zone;
		}
	return $zones;

	} // get_zones

	/*!
		@function get_pointers
		@discussion get pointers based on view, or passed sql
	*/
	function get_pointers($sql = 0) {
		global $conf, $user;

		$pointers = array();

		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, $conf['dbh']);
		while ($p = mysql_fetch_row($db_results)) {
			$pointers[] = new Pointer($p[0]);
		}

	return $pointers;

	} // get_pointers

	function get_vlans ($sql = 0) {
		global $user;

		$vlans = array();
		
		if (!$sql) { $sql = $user->view->get_query(); }
		$db_result = mysql_query($sql, dbh());

		while ( $v = mysql_fetch_object($db_result) ) {
			$vlan = new VLAN($v->id, $v->name, $v->vlan_id);
			$vlans[] = $vlan;
		}

		return $vlans;
	} // get_vlans

	// get an array of the mx records.
	function get_mxs($sql = 0)
	{
		global $conf, $user;
		
		$mxs = array();
		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, $conf['dbh']);
		while ($m = mysql_fetch_object($db_results))
		{
		$mx = new Mx($m->id);
		$mxs[] = $mx;
		}
	
	return $mxs;
	
	} // get_mxs

        // get an array of subnets
        function get_subnets($sql = 0)
        {
                global $conf, $user;

                $mxs = array();
                if (!$sql) { $sql = $user->view->get_query(); }
                $db_results = mysql_query($sql, $conf['dbh']);
                while ($s = mysql_fetch_object($db_results))
                {
                $subnet = new Subnet($s->id);
                $subnets[] = $subnet;
                }

        return $subnets;

        } // get_subnet


	// get an array of user objects
	function get_users($sql = 0) {
		global $conf, $user;

		$users = array();
		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, $conf['dbh']);
		
                while ($user_object = mysql_fetch_object($db_results)) {
                        $temp_user = new User($user_object->username);
                        $results[] = $temp_user;
                }

	return $results;

	} //get_users

	//Get the comments for a specific object
	function get_comments($object_id,$object_type) {
		global $conf;

		$sql = "SELECT * FROM object_history WHERE object_type='$object_type' AND object_id='$object_id'";
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_object($db_results)) {
			$results[] = $r;
		} 

		return $results;

	} //get_comments

	/*!
		@function get_options
		@discussion takes a view or sql statement
			and returns the objects
	*/
	function get_options($sql=0) {
		global $user;

		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, dbh());

		while ($r = mysql_fetch_object($db_results)) {
			$results[] = new Option($r->id);
		}

		return $results;
		
	} // get_options

	/*!
		@function get_admin_options
		@discussion takes a view or sql statement
			and returns adminoption objects
	*/
	function get_admin_options($sql=0) {
		global $user;

		if (!$sql) { $sql = $user->view->get_query(); }
		$db_results = mysql_query($sql, dbh());

		while ($r = mysql_fetch_object($db_results)) { 
			$results[] = new AdminOption($r->id);
		}

		return $results;

	} //get_admin_options

	// display the given help documents
	function show_help($help_docs) {
		global $conf, $user;
		
		// Get total items
		$db_results = mysql_query($user->view->sql, $conf['dbh']);
		$total_items = mysql_num_rows($db_results);
		
		// show the template
		require $conf['prefix'] . "/templates/show_help.inc";
		

	} // show_help

	//mojo for adding/deleteing a zone from a domain
	function modify_domain_zone($domain, $action_type,$zone=0)
	{
		global $conf, $user;
		
		require $conf['prefix'] . "/templates/modify_domain_zones.inc";
	
	
	}

	// This function is for displaying a widget, it depends upon one array and a string
	// $items[] and $title and $width, where width is the size of the table
	function display_widget($items,$title, $width=0)
	{
        
	global $conf, $user;

        if (!$width) { $width="100%"; }
        require $conf['prefix'] . "/templates/widget.inc";
	
	} //display_widget

	/*!
		@function display_top_widget
		@discussion This looks much like tabs in a application.
		@param $items As array()
		@param $width Optional %, default is 100%
		@param $align Optional default is left
	*/
	function display_top_widget($items,$width=0,$align=0) {
		global $conf, $user;

		if (!$width) { $width = "100%"; }
		if (!$align) { $align = "left"; } 

		require $conf['prefix'] . "/templates/top_widget.inc";

	} // display_top_widget


	// This will show a pull down of IP's from start -> end using the form name $name
	// Singular see plural version
	function show_ip_range_select($name, $start, $end) {
		global $conf, $user;

		$ip = new IP(0);

		$count = $start;

		print "<select name=\"$name\">\n";

		while ( $count <= $end ) {
			$calc = $ip->int2ip($count);
			
			print "<option value=\"$count\">$calc</option>\n";
			$count++;
		}
		print "</select>\n";
	} // show_ip_range_select

	// give us a select pulldown with the following ranges and set the $range
	//   as the currently selected one $name is the "name" of the select
	function show_ip_ranges_select ($ranges, $range_id, $name=0) {
		global $conf;


		if (!$name) { $name = "range_id"; }
		// since this is so small let's just put it here
		?>	
			<select name="<?= $name; ?>">
				<? 
				foreach ($ranges as $range) { 
					$used_ips = $range->get_used_ips();
					$total_ips = $range->total_ips;
		
				
				if ( ($range->type == 'dynamic' OR $range->type == 'ddns') AND $used_ips != $total_ips) { 
					$option_text = "(Dynamic Range) ". $range->start->ip_addr ." ($used_ips of $total_ips used) ". $range->default_domain->name;
				}
				elseif ( ($range->type == 'dynamic' OR $range->type == 'ddns') AND $used_ips >= $total_ips) { 
					$option_text = "(Dynamic Range) ". $range->start->ip_addr ." FULL";
					if ($used_ips > $total_ips) {
						$overfilled = $used_ips - $total_ips;
						$option_text .= " Overfilled by $overfilled";
					}
				}
				elseif ( $range->type == 'static' AND $used_ips >= $total_ips ) {
					$option_text = $range->start->ip_addr ." to ". $range->end->ip_addr ." (FULL) ". $range->default_domain->name;
				}
				else { 
					$option_text = $range->start->ip_addr ." to ". $range->end->ip_addr ." ($used_ips of $total_ips used) ". $range->default_domain->name;
				}
				?>
				<option value="<?= $range->id ?>" <? if ( $range->id == $range_id ) { echo "SELECTED"; } ?>> 
				<?= $option_text; ?>
				</option>
				<? } //end foreach?>
			</select>
		<?
	}

	// do some subnet mojo
	function show_subnet ($action, $subnet) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_subnet.inc";
	}

	/*!
		@function show_pointers
		@discussion show pointers based on $pointers array
	*/
	function show_pointers($action, $pointers) {
		global $conf, $user;

		require ($conf['prefix'] . "/templates/show_pointers.inc");

	} // show_pointers

	// show all of the subnets
	function show_subnets ($action, $subnets) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_subnets.inc";
	}

	function show_unused_subnets ($subnets) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_unused_subnets.inc";
	}

		// do some subnet mojo
	function show_workgroup ($action, $workgroup) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_workgroup.inc";
	}

	// show all of the subnets
	function show_workgroups ($action, $workgroups) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_workgroups.inc";
	}

	/*!
		@function show_workgroup_zones 
		@discussion shows the mapping between workgroups and zones. 
	*/
	function show_workgroup_zones ($workgroup) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_workgroup_zones.inc";
		
	} // show_workgroup_zones

	// $type is either workgroup or subnet and option is an Option object
	function show_option ($action, $type, $type_id, $option) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_option.inc";
	}

	// $type is either workgroup or subnet and option is an Option object
	function show_options ($action, $type, $object_id, $options) {
		global $conf, $user;

		require $conf['prefix'] . "/templates/show_options.inc";
	}

	// This will show a pull down of IP's from start -> end using the form name $name
	function show_dhcp_option_select ($option_id,$name) {
		global $conf, $user;

		if (!$name) { $name = "dhcp_option_id"; }

		$sql = "SELECT * FROM dhcp_options ORDER BY name";
		$db_result = mysql_query($sql, $conf['dbh']);

		$option = new Option($option_id);

		print "<select name=\"$name\">\n";
	
		while ( $obj = mysql_fetch_object($db_result) ) {
			if ( $obj->id == $option_id) {
				print "\t<option value=\"$obj->id\" selected>" . $obj->name . "</option>\n";
			}
			else {
				print "\t<option value=\"$obj->id\">$obj->name</option>\n";
			}
		}

		print "</select>\n";

	} // show_dhcp_option_select

	/*!
		@function show_subnect_select
		@discussion This will show a pull down of subnets, all for admin, and only ones that
			zone admins have access to...
	*/
	function show_subnet_select ($subnet_id) {
		global $conf, $user;

		// Show the admin subnet selecter
		if ($user->is_admin) { 
			$sql = "SELECT * FROM subnet ORDER BY subnet";
			$db_result = mysql_query($sql, $conf['dbh']);

			while ( $obj = mysql_fetch_object($db_result) ) {
				$subnet = new Subnet($obj->id);
				$subnets[] = $subnet;
			}

		} // end if admin

		else {
			$ranges = $conf['zone']->ip_ranges();
			foreach ($ranges as $range) {
				$subnets[] = new Subnet($range->subnet);
			}
		} // if not an admin

		print "<select name=\"subnet_id\">\n";

		//Recurse through the subnets we got from above..
		foreach ($subnets as $subnet) {

			$subnet_mask = $subnet->subnet->ip_addr;
			$netmask     = $subnet->netmask->ip_addr;
			
			if ( $subnet->id == $subnet_id ) {
				print "<option value=\"$subnet->id\" SELECTED>$subnet->name - ($subnet_mask/$netmask)</option>\n";
			}
			else {
				print "<option value=\"$subnet->id\">$subnet->name ($subnet_mask/$netmask)</option>\n";
			}
		} // end foreach

		print "</select>\n";


	} // show_subnet_select


	// This will show a pull down of IP's from start -> end using the form name $name
	function show_vlan_select ($vlan_id) {
		global $conf, $user;

		$sql = "SELECT * FROM vlan ORDER BY name";
		$db_result = mysql_query($sql, $conf['dbh']);
		
		echo "<select name=\"vlan_id\">\n";
		echo "<option value=\"0\">No Vlan</option>\n";

		while ( $obj = mysql_fetch_object($db_result) ) {

			if ( $obj->id == $vlan_id ) {
				print "<option value=\"$obj->id\" SELECTED>$obj->name</option>\n";
			}
			else {
				print "<option value=\"$obj->id\">$obj->name</option>\n";
			}
		}

		echo "</select>\n";
	} // show_vlan_select


	// show the SOA records listing
	function show_soa_records ($soas) {
		global $user, $conf;

		// get the total number of soa records here
		$total_items = count_soa();

		// show the template
		require $conf['prefix'] . "/templates/show_soa_records.inc";
	}

	// Just show one soa
	function show_soa ($soa, $action) {
		global $user, $conf;

		// show the template
		require $conf['prefix'] . "/templates/show_soa.inc";
	}

	/*!
		@function get_zone_owner_email
		@discussion takes a list of zones and returns a listing
			of all the owners, or the support e-mail address
	*/
	function get_zone_owner_email($zones) {
		global $conf, $user;


                $sql = "SELECT DISTINCT(zone_user.username) FROM zone_user WHERE level='50' AND (";
		
                if (is_array($zones)) {
                        foreach ($zones as $zone) {
					
				if ($zone->email) { $results[] = $zone->email; } 
                                elseif (!$add) { $sql .= " zone='" . $zone->id . "' "; $add=1;}
                                else { $sql .= " OR zone='" . $zone->id . "' "; }
                        }
                } // is array
                else {
			if ($zones->email) { $results[] = $zone->email; }
			else { $sql .= " zone='" . $zones->id . "'"; $add=1;}
                } // end else
                
		$sql .= ")";

		if ($add) {
			$db_results = mysql_query($sql, dbh());
		}

		while ($r = mysql_fetch_object($db_results)) {
			
			$results[] = $r->username . "@onid.oregonstate.edu";

		} // end while

		return $results;

	} // get_zone_owner_email

	/*!
		@function notify_admins
		@discussion takes text, subject and a list of zones
			and figures out who we need to send the e-mail
			to.
	*/
        function notify_admins($text,$subject,$zones) {
                global $conf, $user;		

		$results = $this->get_zone_owner_email($zones);

		foreach($results as $email) {
			if ($to) { $to .= ", $email"; }
			else { $to = "$email"; }
		}

		if ($conf['debug']) { $to = "vollmerk@net.orst.edu"; }
                
		mail ($to,"[Maintain-Update] " . $subject,$text,
                        "From: maintain@net.oregonstate.edu\r\n"
                        ."Reply-To: maintain@net.oregonstate.edu\r\n"
                        ."CC: maintain@net.oregonstate.edu\r\n");

        } // notify_admins      

} // end ui class

?>
